#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>

#include "common.h"
#include "macros.h"
#include "io.h"
#include "misc.h"
#include "timeout.h"
#include "signal-event.h"
#include "hal-proto.h"
#include "hal-mgr.h"
#include "sock-inet.h"
#include "sock-unix.h"

#define info(fmt, arg...) fprintf(stderr, "info: %s: "fmt CRLF, \
				__FUNCTION__, ##arg);

#define error(eval, fmt, arg...)			\
do {							\
	fprintf(stderr, "error: %s: "fmt CRLF,		\
				__FUNCTION__, ##arg);	\
	exit(eval);					\
} while (0)

static struct io_context *io;
static int terminate;

static void handle_signals(int signo, siginfo_t *si, void *opaque)
{
	switch (signo) {
	case SIGINT:
	case SIGTERM:
		info("exiting by signal %d", signo);
		terminate = 1;
		io_loop_stop(io);
		break;
	}
}

int main(int argc, char **argv)
{
	int rfd, wfd, sfd, port;
	char buf[32];
	ret_t ret;

	port = 4000;

	if (argc < 2)
		error(1, "not enough arguments: write fifo read fifo");

	/* Open pipes and get file descriptors. */
	ret = pipe_open(argv[1], O_RDWR, &wfd);
	if (ret != ret_ok)
		error(1, "can't open write pipe");

	ret = pipe_open(argv[2], O_RDWR, &rfd);
	if (ret != ret_ok)
		error(1, "can't open read pipe");

	/* Set signal handlers. */
	sigevent_ignore(SIGPIPE);
	sigevent_add(SIGINT, handle_signals, NULL, 1);
	sigevent_add(SIGTERM, handle_signals, NULL, 1);
	sigevent_init();

	/* Init loop facilities. */
	timeout_init();
	io = io_new();

	snprintf(buf, sizeof(buf), "%d",  port);
	sfd = sock_inet_connect_wait("tcp", "ip", "192.168.1.100", buf, 0);
	if (sfd == -1)
		error(1, "can't connect to server");

	sock_nonblock(sfd);

	/* Init manager and add a runtime client. */
	hal_mgr_init(io);
	hal_mgr_client_raw_new("vmachined", wfd, rfd, 0, HAL_CLT_RUNTIME);
	hal_mgr_client_raw_new("lincond", sfd, sfd, 0, HAL_CLT_SLAVE);

	hal_mgr_clients_connect();

	while (!terminate)
		io_loop_run(io);

	sock_inet_close(sfd);
	/* Free resources. */
	hal_mgr_deinit();
	io_free(&io);
	timeout_deinit();
	sigevent_deinit();

	return 0;
}

